/* gnubiff -- a mail notification program
 * Copyright (C) 2000-2003 Nicolas Rougier
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this software; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */
#include <string>
#include <sstream>
#include <glib.h>
#include <sys/stat.h>
#include <utime.h>
#include <dirent.h>
#include <time.h>

#include "Maildir.h"

// ===================================================================
// = Static attributes ===============================================
// ===================================================================


// ===================================================================
// = Callbacks "C" binding ===========================================
// ===================================================================


// ===================================================================
// = Maildir =========================================================
// ===================================================================
Maildir::Maildir (Biff *owner) : Mailbox (owner) {
}


// ===================================================================
// = ~Maildir ========================================================
// ===================================================================
Maildir::~Maildir (void) {
}

// ===================================================================
// = update ==========================================================
// ===================================================================
void Maildir::update (void) {
  static time_t m_time=0;
  struct stat file_stat;
  DIR *dir;
  struct dirent *dent;
  int dirsize=0;

  // Default behavior
  _status = MAILBOX_ERROR;

  string directory = _owner->_address + "/new";
  // Check for existence of a new mail directory
  if ((stat (directory.c_str(), &file_stat) != 0)||(!S_ISDIR(file_stat.st_mode)))
    return;

  // Try to open new mail directory
  if ((dir = opendir (directory.c_str())) == NULL)
    return;

  // Read number of entries (but '.'  and '..')
  while ((dent = readdir(dir)))
    if (dent->d_name[0] != '.')
      dirsize++;
  closedir (dir); 
   
  // No entry  = no new mail
  if (dirsize == 0)
    _status = MAILBOX_EMPTY;
  else {
    // New mail directory has not been modified
    if (file_stat.st_mtime == m_time)
      _status = MAILBOX_OLD;
    // New mail directory has been modified
    else
      _status = MAILBOX_NEW;
  }

  // Save new modification time for next time
  m_time = file_stat.st_mtime;
}


// ===================================================================
// = fetch ===========================================================
// ===================================================================
void Maildir::fetch (void) {
  DIR *dir;
  struct stat file_stat;
  struct dirent *dent;
  int saved_status = _status;
  
  // Status will be restored in the end if no problem occured
  _status = MAILBOX_ERROR;

  string directory = _owner->_address + "/new";
  // Check for existence of a new mail directory
  if ((stat (directory.c_str(), &file_stat) != 0)||(!S_ISDIR(file_stat.st_mode))) {
    g_warning ("Cannot find new mail directory (%s)", directory.c_str());
    return;
  }

  // Try to open new mail directory
  if ((dir = opendir (directory.c_str())) == NULL) {
    g_warning ("Cannot open new mail directory (%s)", directory.c_str());
    return;
  }

  _unreads.clear();
  vector<string> mail;
  string line; 
  // Read new mails
  while ((dent = readdir(dir)) && (_unreads.size() < _owner->_max_header_display)) {
    if (dent->d_name[0]=='.')
      continue;
    ifstream file;
    string filename = directory + string("/") + string (dent->d_name);
    file.open (filename.c_str());
    if (file.is_open()) {
      while (!file.eof()) {
	string line;
	getline(file, line);
	mail.push_back (line);
      }
      parse (mail);
      mail.clear();
    }
    else {
      g_warning ("Cannot open %s.", filename.c_str());
    }
    file.close();
  }
  closedir (dir);

  // Restore status
  _status = saved_status;
}
